//csc.exe /target:winexe /optimize unmark.cs
//.NET 4.8
using System;
using System.ComponentModel;
using System.Windows.Forms;
using System.Drawing;
using System.IO;
using System.Text;
using System.Globalization;
using System.Collections.Generic;
using System.Net;
//using System.Net.Security;
using System.Threading;

namespace AwemerApp {
	public struct Aweme {
		public string Filename;
		public bool Improve;
	}
	/*public class UnexpectedException: Exception {
		public UnexpectedException() {}
		public UnexpectedException(string message): base(message) {}
		public UnexpectedException(string message, Exception inner): base(message, inner) {}
	}*/
	public class AwemeWebClient: WebClient {
		protected override WebResponse GetWebResponse(WebRequest request) {
			(request as HttpWebRequest).AllowAutoRedirect = true;
			HttpWebResponse r = base.GetWebResponse(request) as HttpWebResponse;
			if (r.ContentLength == 0) {
				throw new Exception("Response body is empty.");
			}
			return r;
		}
		/*public bool DownloadNonEmptyFile(Uri uri, string filename, byte[] buffer) {
			int i;
			byte[] b;
			Stream s;
			FileStream f;
			using (s = OpenRead(uri)) {
				if ((i = s.ReadByte()) == -1) {
					return false;
				}
				b = buffer != null ? buffer : new byte[1024];
				using (f = new FileStream(filename, FileMode.Create)) {
					f.WriteByte((byte)i);
					while ((i = s.Read(b, 0, b.Length)) > 0) {
						f.Write(b, 0, i);
					}
					f.Flush();
				}
			}
			return true;
		}
		public bool DownloadNonEmptyFile(string uri, string filename, byte[] buffer) {
			return DownloadNonEmptyFile(new Uri(uri), filename, buffer);
		}*/
	}
	public class MainForm: Form {
		public const string DIRECTORY = "download";
		public string Root;
		public bool Stop;
		public bool Quality;
		public int Count;
		public Graphics Graphics = null;
		public IContainer Components = new Container();
		public ToolTip Tip = new ToolTip();
		public FolderBrowserDialog DirOpener = new FolderBrowserDialog();
		public CheckBox ImproveQuality = new CheckBox();
		public Label DirLabel = new Label();
		public TextBox DirInput = new TextBox();
		public Button OpenDir = new Button();
		public Button Download = new Button();
		public TextBox Output = new TextBox();
		public readonly object StopLock = new Object();
		public readonly object OutLock = new Object();
		public readonly object SyncLock = new Object();
		public MainForm(string root) {
			Root = root;
			SuspendLayout();
			Components.Add(Tip);
			Components.Add(DirOpener);
			Graphics = CreateGraphics();
			int t, w = (int)Graphics.MeasureString("directory:", DirLabel.Font).Width + 5;
			DirOpener.RootFolder = Environment.SpecialFolder.Desktop;
			DirOpener.SelectedPath = Root;
			ImproveQuality.AutoSize = false;
			ImproveQuality.Size = new Size((int)Graphics.MeasureString("improve quality", ImproveQuality.Font).Width + 24, 21);
			ImproveQuality.Location = new Point(5, t = 5);
			ImproveQuality.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			ImproveQuality.Text = "improve quality";
			ImproveQuality.CheckedChanged += ImproveQualityChanged;
			DirLabel.AutoSize = false;
			DirLabel.Size = new Size(w, 21);
			DirLabel.Location = new Point(5, t += 5 + 21);
			DirLabel.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			DirLabel.Text = "directory:";
			DirLabel.TextAlign = ContentAlignment.MiddleLeft;
			DirInput.Size = new Size(210 - 26, 21);
			DirInput.Location = new Point(2 * 5 + w, t);
			DirInput.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Left);
			OpenDir.Size = new Size(21, 21);
			OpenDir.Location = new Point(2 * 5 + w + 210 - 21, t);
			OpenDir.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right);
			OpenDir.Text = "⯇";
			OpenDir.Click += OpenDirClick;
			w = 3 * 5 + w + 210;
			Download.Size = new Size(90, 21);
			Download.Location = new Point((w - 90) / 2, t += 5 + 21);
			Download.Anchor = (AnchorStyles)(AnchorStyles.Top);
			Download.Text = "download";
			Download.Click += DownloadClick;
			Output.Size = new Size(w - 2 * 5, 210);
			Output.Location = new Point(5, t += 5 + 21);
			Output.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Bottom | AnchorStyles.Left);
			Output.Multiline = true;
			Output.ReadOnly = true;
			Output.BackColor = Color.FromKnownColor(KnownColor.Window);
			Output.WordWrap = false;
			Output.ScrollBars = ScrollBars.Both;
			Tip.ShowAlways = true;
			Tip.AutoPopDelay = 9000;
			Tip.InitialDelay = 300;
			Tip.ReshowDelay = 300;
			Tip.SetToolTip(DirInput, "directory with watermarked videos from before 2020-07-27");
			Tip.SetToolTip(Output, "output");
			ClientSize = new Size(w, t + 5 + 210);
			Text = "unmark";
			ShowIcon = false;
			Controls.Add(ImproveQuality);
			Controls.Add(DirLabel);
			Controls.Add(DirInput);
			Controls.Add(OpenDir);
			Controls.Add(Download);
			Controls.Add(Output);
			ResumeLayout(false);
			PerformLayout();
			Show();
			MinimumSize = Size;
			DirInput.Focus();
		}
		protected override void Dispose(bool disposing) {
			if (disposing) {
				if (Graphics != null) {
					Graphics.Dispose();
				}
				if (Components != null) {
					Components.Dispose();
				}
			}
			base.Dispose(disposing);
		}
		public void DisableControls() {
			Stop = false;
			Download.Text = "stop";
			ImproveQuality.Enabled = false;
			DirInput.ReadOnly = true;
			OpenDir.Enabled = false;
			Cursor.Current = Cursors.WaitCursor;
		}
		public void EnableControls() {
			Stop = false;
			Download.Text = "download";
			ImproveQuality.Enabled = true;
			DirInput.ReadOnly = false;
			OpenDir.Enabled = true;
			Cursor.Current = Cursors.Default;
		}
		public static string GetFullDir(string directory, string root) {
			if (directory.Length == 0) {
				return root;
			}
			string t;
			switch (directory[directory.Length - 1]) {
				case '/':
				case '\\':
					if (directory.Length == 1) {
						return root;
					}
					t = directory.Substring(0, directory.Length - 1);
					break;
				default:
					t = directory;
					break;
			}
			return Path.GetFullPath(Path.IsPathRooted(t) ? t : root + Path.DirectorySeparatorChar + t);
		}
		/*public static string GeneratePublicIp() {
			int i0, i1, i2;
			Random r = new Random();
			while ((i0 = r.Next(1, 224)) == 10 || i0 == 127) {}
			switch (i0) {
				case 100:
					while ((i1 = r.Next(256)) > 63 && i1 < 128) {}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 169:
					while ((i1 = r.Next(256)) == 254) {}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 172:
					while ((i1 = r.Next(256)) > 15 && i1 < 32) {}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 192:
					while ((i1 = r.Next(256)) == 168) {}
					switch (i1) {
						case 0:
							while ((i2 = r.Next(1, 256)) == 2) {}
							return i0.ToString() + "." + i1.ToString() + "." + i2.ToString() + "." + r.Next(256).ToString();
						case 88:
							while ((i2 = r.Next(256)) == 99) {}
							return i0.ToString() + "." + i1.ToString() + "." + i2.ToString() + "." + r.Next(256).ToString();
					}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 198:
					while ((i1 = r.Next(256)) == 18 || i1 == 19) {}
					if (i1 == 51) {
						while ((i2 = r.Next(256)) == 100) {}
						return i0.ToString() + "." + i1.ToString() + "." + i2.ToString() + "." + r.Next(256).ToString();
					}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 203:
					if ((i1 = r.Next(256)) == 0) {
						while ((i2 = r.Next(256)) == 113) {}
						return i0.ToString() + "." + i1.ToString() + "." + i2.ToString() + "." + r.Next(256).ToString();
					}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
			}
			return i0.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
		}*/
		/*public static string GenerateIp() {
			Random r = new Random();
			return r.Next(0, 256).ToString() + '.' + r.Next(0, 256).ToString() + '.' + r.Next(0, 256).ToString() + '.' + r.Next(0, 256).ToString();
		}*/
		public static string GetVid(string filename) {
			int i, b;
			byte[] v = new byte[64];
			using (FileStream s = File.Open(filename, FileMode.Open, FileAccess.Read, FileShare.Read)) {
				if ((b = s.ReadByte()) == -1) {
					return null;
				}
				do {
					if (b == 'v') {
						if ((b = s.ReadByte()) == 'i' && (b = s.ReadByte()) == 'd' && (b = s.ReadByte()) == ':') {
							for (i = 0; i < v.Length && (b = s.ReadByte()) > 0; i++) {
								v[i] = (byte)b;
							}
							return Encoding.UTF8.GetString(v, 0, i);
						}
					} else {
						b = s.ReadByte();
					}
				} while (b > -1);
			}
			return null;
		}
		public void DownloadAweme(object aweme) {
			Aweme a = (Aweme)aweme;
			try {
				int r = 0;
				string n, u;
				if (File.Exists(n = DIRECTORY + Path.DirectorySeparatorChar + a.Filename) || (u = GetVid(a.Filename)) == null) {
					return;
				}
				AwemeWebClient c = new AwemeWebClient();
				u = "https://api.tiktokv.com/aweme/v1/play/?video_id=" + (a.Improve ? u + "&improve_bitrate=1&ratio=default" : u);
				do {
					try {
						c.Headers.Set("User-Agent", "okhttp");
						c.DownloadFile(u, n);
						if ((new FileInfo(n)).Length == 0) {
							File.Delete(n);
							throw new Exception("File is empty.");
						}
						break;
					} catch (Exception) {
						if (r == 3) {
							File.Delete(n);
							throw;
						}
						r++;
					}
				} while (true);
				lock (OutLock) {
					Output.AppendText(a.Filename);
					Output.AppendText(" is downloaded");
					Output.AppendText(Environment.NewLine);
				}
			} catch (Exception x) {
				lock (OutLock) {
					Output.AppendText(a.Filename);
					Output.AppendText(" failed");
					Output.AppendText(Environment.NewLine);
					Output.AppendText(x.InnerException != null ? x.InnerException.Message : x.Message);
					Output.AppendText(Environment.NewLine);
				}
			} finally {
				lock (SyncLock) {
					Count--;
					Monitor.Pulse(SyncLock);
				}
			}
		}
		public void DownloadVideos(object directory) {
			try {
				string f;
				IEnumerable<string> d = Directory.EnumerateFiles((string)directory);
				foreach (string e in d) {
					if (Path.GetExtension(f = Path.GetFileName(e)).ToLower() == ".mp4" && !File.Exists(DIRECTORY + Path.DirectorySeparatorChar + f)) {
						lock (SyncLock) {
							Count++;
						}
						new Thread(new ParameterizedThreadStart(DownloadAweme)).Start(new Aweme {
							Filename = f,
							Improve = Quality
						});
						lock (SyncLock) {
							if (Count >= 10) {
								Monitor.Wait(SyncLock);
							}
						}
					} else {
						lock (StopLock) {
							if (Stop) {
								break;
							}
						}
					}
				}
			} catch (Exception x) {
				lock (OutLock) {
					Output.AppendText(x.Message);
					Output.AppendText(Environment.NewLine);
				}
			}
			do {
				lock (SyncLock) {
					if (Count == 0) {
						break;
					}
					Monitor.Wait(SyncLock);
				}
			} while (true);
			EnableControls();
		}
		public void ImproveQualityChanged(object sender, EventArgs args) {
			Quality = ImproveQuality.Checked;
		}
		public void OpenDirClick(object sender, EventArgs args) {
			if (DirOpener.ShowDialog() != DialogResult.OK) {
				return;
			}
			DirInput.Text = DirOpener.SelectedPath;
		}
		public void DownloadClick(object sender, EventArgs args) {
			lock (StopLock) {
				Stop = true;
				if (DirInput.ReadOnly) {
					return;
				}
			}
			DisableControls();
			try {
				string d = GetFullDir(DirInput.Text, Root);
				if (!Directory.Exists(d)) {
					MessageBox.Show("Directory does not exists.");
					EnableControls();
				}
				Directory.SetCurrentDirectory(d);
				if (!Directory.Exists(DIRECTORY)) {
					Directory.CreateDirectory(DIRECTORY);
				}
				Output.Text = "";
				new Thread(new ParameterizedThreadStart(DownloadVideos)).Start(d);
			} catch (Exception x) {
				MessageBox.Show(x.Message);
				EnableControls();
			}
		}
		/*public static bool ValidateRemoteCertificate(object sender, object certificate, object chain, SslPolicyErrors sslPolicyErrors) {
			return true;
		}*/
		[STAThread]
		public static void Main() {
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);
			ServicePointManager.SecurityProtocol |= SecurityProtocolType.Ssl3 | SecurityProtocolType.Tls | SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12 | SecurityProtocolType.Tls13;//.NET 4.8
			//ServicePointManager.ServerCertificateValidationCallback += ValidateRemoteCertificate;//.NET 3.5
			Application.Run(new MainForm(Directory.GetCurrentDirectory()));
		}
	}
}